﻿<#
.SYNOPSIS
	This script performs the installation or uninstallation of an application(s).
.DESCRIPTION
	The script is provided as a template to perform an install or uninstall of an application(s).
	The script either performs an "Install" deployment type or an "Uninstall" deployment type.
	The install deployment type is broken down into 3 main sections/phases: Pre-Install, Install, and Post-Install.
	The script dot-sources the AppDeployToolkitMain.ps1 script which contains the logic and functions required to install or uninstall an application.
.PARAMETER DeploymentType
	The type of deployment to perform. Default is: Install.
.PARAMETER DeployMode
	Specifies whether the installation should be run in Interactive, Silent, or NonInteractive mode. Default is: Interactive. Options: Interactive = Shows dialogs, Silent = No dialogs, NonInteractive = Very silent, i.e. no blocking apps. NonInteractive mode is automatically set if it is detected that the process is not user interactive.
.PARAMETER AllowRebootPassThru
	Allows the 3010 return code (requires restart) to be passed back to the parent process (e.g. SCCM) if detected from an installation. If 3010 is passed back to SCCM, a reboot prompt will be triggered.
.PARAMETER TerminalServerMode
	Changes to "user install mode" and back to "user execute mode" for installing/uninstalling applications for Remote Destkop Session Hosts/Citrix servers.
.PARAMETER DisableLogging
	Disables logging to file for the script. Default is: $false.
.EXAMPLE
	Deploy-Application.ps1
.EXAMPLE
	Deploy-Application.ps1 -DeployMode 'Silent'
.EXAMPLE
	Deploy-Application.ps1 -AllowRebootPassThru -AllowDefer
.EXAMPLE
	Deploy-Application.ps1 -DeploymentType Uninstall
.NOTES
	Toolkit Exit Code Ranges:
	60000 - 68999: Reserved for built-in exit codes in Deploy-Application.ps1, Deploy-Application.exe, and AppDeployToolkitMain.ps1
	69000 - 69999: Recommended for user customized exit codes in Deploy-Application.ps1
	70000 - 79999: Recommended for user customized exit codes in AppDeployToolkitExtensions.ps1
    
    The script will install x86 and x64 components of the EM-Agent if they are stored in the script directory. But i do not recommend to deploy both archtectures in one package, because of duplicate donwload for both agents. 
    Recommendation is, to create two seperate applications or two deployment-Types in one application. 
.LINK 
	http://psappdeploytoolkit.com
.CHANGENOTES
    - 2.0.0 Added unblock and automatic start of Agent and Service in the Post-Installation section of the script
    - 3.0.0 Added the possibility to configure the start behavior of the service (delayed start or normal start)
    - 3.1.0 Catched exception when it failes to read out a registry key
    - 3.3.0 Moved backend function to an external script (AppDeployToolkitExtensions.Agent.ps1). Integrated support for x86 and x64 plattform. 
#>
[CmdletBinding()]
Param (
	[Parameter(Mandatory=$false)]
	[ValidateSet('Install','Uninstall')]
	[string]$DeploymentType = 'Install',
	[Parameter(Mandatory=$false)]
	[ValidateSet('Interactive','Silent','NonInteractive')]
	[string]$DeployMode = 'Interactive',
	[Parameter(Mandatory=$false)]
	[switch]$AllowRebootPassThru = $false,
	[Parameter(Mandatory=$false)]
	[switch]$TerminalServerMode = $false,
	[Parameter(Mandatory=$false)]
	[switch]$DisableLogging = $false,
    [Parameter(Mandatory=$true)]
	[string]$ServerName = '',
    [Parameter(Mandatory=$false)]
	[string]$ServerPort = '9010',
    [Parameter(Mandatory=$false)]
	[int]$DelayedAutostart = 0,
    [Parameter(Mandatory=$false)]
    [int]$ActionTimeout=10

)

Try {
	## Set the script execution policy for this process
	Try { Set-ExecutionPolicy -ExecutionPolicy 'ByPass' -Scope 'Process' -Force -ErrorAction 'Stop' } Catch {}
	
	##*===============================================
	##* VARIABLE DECLARATION
	##*===============================================
	## Variables: Application
	[string]$appVendor = 'Matrix42'
	[string]$appName = 'Enterprise Manager Agent'
	[string]$appVersion = '20.0.0.20097'
	[string]$appArch = 'x64x86'
	[string]$appLang = 'ENDE'
	[string]$appRevision = '1'
	[string]$appScriptVersion = '3.3.0'
	[string]$appScriptDate = '06/04/2020'
	[string]$appScriptAuthor = 'Hesse, Jannik'
	##*===============================================
	
	##* Do not modify section below
	#region DoNotModify
	
	## Variables: Exit Code
	[int32]$mainExitCode = 0
	
	## Variables: Script
	[string]$deployAppScriptFriendlyName = 'Deploy Application'
	[version]$deployAppScriptVersion = [version]'3.6.5'
	[string]$deployAppScriptDate = '08/17/2015'
	[hashtable]$deployAppScriptParameters = $psBoundParameters
	
	## Variables: Environment
	If (Test-Path -LiteralPath 'variable:HostInvocation') { $InvocationInfo = $HostInvocation } Else { $InvocationInfo = $MyInvocation }
	[string]$scriptDirectory = Split-Path -Path $InvocationInfo.MyCommand.Definition -Parent
	
	## Dot source the required App Deploy Toolkit Functions
	Try {
		[string]$moduleAppDeployToolkitMain = "$scriptDirectory\AppDeployToolkit\AppDeployToolkitMain.ps1"
		If (-not (Test-Path -LiteralPath $moduleAppDeployToolkitMain -PathType 'Leaf')) { Throw "Module does not exist at the specified location [$moduleAppDeployToolkitMain]." }
		If ($DisableLogging) { . $moduleAppDeployToolkitMain -DisableLogging } Else { . $moduleAppDeployToolkitMain }
	}
	Catch {
		If ($mainExitCode -eq 0){ [int32]$mainExitCode = 60008 }
		Write-Error -Message "Module [$moduleAppDeployToolkitMain] failed to load: `n$($_.Exception.Message)`n `n$($_.InvocationInfo.PositionMessage)" -ErrorAction 'Continue'
		## Exit the script, returning the exit code to SCCM
		If (Test-Path -LiteralPath 'variable:HostInvocation') { $script:ExitCode = $mainExitCode; Exit } Else { Exit $mainExitCode }
	}

    Try {
		[string]$moduleEMAgentExtension = "$scriptDirectory\AppDeployToolkit\AppDeployToolkitExtensions.Agent.ps1"
		If (-not (Test-Path -LiteralPath $moduleEMAgentExtension -PathType 'Leaf')) { Throw "Module does not exist at the specified location [$moduleEMAgentExtension]." }
		If ($DisableLogging) { . $moduleEMAgentExtension -DisableLogging } Else { . $moduleEMAgentExtension }
	}
	Catch {
		If ($mainExitCode -eq 0){ [int32]$mainExitCode = 60008 }
		Write-Error -Message "Module [$moduleEMAgentExtension] failed to load: `n$($_.Exception.Message)`n `n$($_.InvocationInfo.PositionMessage)" -ErrorAction 'Continue'
		## Exit the script, returning the exit code to SCCM
		If (Test-Path -LiteralPath 'variable:HostInvocation') { $script:ExitCode = $mainExitCode; Exit } Else { Exit $mainExitCode }
	}
	
	#endregion
	##* Do not modify section above
	##*===============================================
	##* END VARIABLE DECLARATION
	##*===============================================
    
    #Wait until agent has no open jobs
    Wait-EMAgentAvailability -ActionTimeout $ActionTimeout

    Stop-ServiceAndDependencies -Name 'M42EmAgent'
    Show-InstallationWelcome -CloseApps 'EmAgentGui=EM Agent GUI,EMAgent=EM Agent' -CloseAppsCountdown 1 -BlockExecution

    # ATTENTION: It is not possible to use the build-in function of PSADK to uninstall old version here. 
    #            The reason is, that the uninstall string is not being written to the registry with the EM-Agent Installer.
    $UninstallPath = "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall" 
    $SearchString = "Matrix42 Enterprise Manager 6.6 Agent", "Matrix42 Enterprise Manager 7.3 Agent", "Matrix42 Enterprise Manager 19.0.2 Agent", "Matrix42 Enterprise Manager 19.0.3 Agent", "Matrix42 Enterprise Manager 20.0.0 Agent"

    foreach($ProductString in $SearchString) 
    {
        $MsiGuid = (Get-WmiObject Win32_Product | Where-Object { $_.Name -eq $ProductString }).IdentifyingNumber
        if($MsiGuid -ne $null)
        {
            Write-Log -Message "Product ""$ProductString"" found in WMI Database with GUID ""$MsiGuid"". Start Uninstall." -Source "EM-AgentPackage" -LogType "CMTrace" 
            Execute-MSI -Action Uninstall -Path "$MsiGuid"  
        }
        else 
        { 
            Write-Log -Message "Product ""$ProductString"" not found in registry." -Source "EM-AgentPackage" -LogType "CMTrace"
        }
    }

	If ($deploymentType -ine 'Uninstall') {
		##*===============================================
		##* PRE-INSTALLATION
		##*===============================================
		[string]$installPhase = 'Pre-Installation'
		
		$params = "/NORESTART /QB EMSERVER={0} EMPORT={1} START=0 OSD=1" -f $ServerName, $ServerPort
		
		##*===============================================
		##* INSTALLATION 
		##*===============================================
		[string]$installPhase = 'Installation'
		
		## Handle Zero-Config MSI Installations
		If ($useDefaultMsi) {
			[hashtable]$ExecuteDefaultMSISplat =  @{ Action = 'Install'; Path = $defaultMsiFile }; If ($defaultMstFile) { $ExecuteDefaultMSISplat.Add('Transform', $defaultMstFile) }
			Execute-MSI @ExecuteDefaultMSISplat; If ($defaultMspFiles) { $defaultMspFiles | ForEach-Object { Execute-MSI -Action 'Patch' -Path $_ } }
		}

        ##Start setup depending on plattform.
        if($envArchitecture -eq "AMD64") {
		    if(!(Test-Path "$dirFiles\Matrix42 Enterprise Manager Agent(x64).msi")) {
                # Skip installation. It is technical possible to run x86 agent on x64 plattform, but not official supported. If you want to do, please change by yourself and on your own risk. 
                Write-Log -Message "Running on x64 plattform, but x64 components are not integrated into package" -Source "EM-AgentPackage"
            } else {
                Execute-MSI -Action Install -Path 'Matrix42 Enterprise Manager Agent(x64).msi' -Parameters $params 
            }
        }
        else {
		    if(!(Test-Path "$dirFiles\Matrix42 Enterprise Manager Agent(x86).msi")) {
                Write-Log -Message "Running on x86 plattform, but x86 components are not integrated into package" -Source "EM-AgentPackage"
            } else {
                Execute-MSI -Action Install -Path 'Matrix42 Enterprise Manager Agent(x86).msi' -Parameters $params 
            }
        }

		##*===============================================
		##* POST-INSTALLATION
		##*===============================================
		[string]$installPhase = 'Post-Installation'
		

        # Configure Autostart behavior
        if($DelayedAutostart -eq $true)
        {
            Write-Log -Message "Activating delayed start of the service" -Source "EM-AgentPackage"
            Set-RegistryKey -Key "HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\M42EmAgent" -Name 'DelayedAutostart' -Value 1
        }
        else
        {
            Write-Log -Message "Activating normal start of the service" -Source "EM-AgentPackage"
            Set-RegistryKey -Key "HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\M42EmAgent" -Name 'DelayedAutostart' -Value 0
        }

        # Unblock App Execution of AgentGui and AgentService
        Unblock-AppExecution

        # Start EM-Agent Service
        Start-ServiceAndDependencies -Name 'M42EmAgent'

        # Start EM-Agent Frontend
        Write-Log -Message "Try to start EM-Agent from ""$envProgramFiles""" -Source "EMAgent-Package"
        Start-Process -FilePath "$envProgramFiles\Matrix42\Matrix42 Enterprise Manager Agent\EmAgentGui.exe"

		## <Perform Post-Installation tasks here>
		Register-Installation
	}
	ElseIf ($deploymentType -ieq 'Uninstall')
	{
		##*===============================================
		##* PRE-UNINSTALLATION
		##*===============================================
		[string]$installPhase = 'Pre-Uninstallation'
		
		## <Perform Pre-Uninstallation tasks here>
		
		
		##*===============================================
		##* UNINSTALLATION
		##*===============================================
		[string]$installPhase = 'Uninstallation'
		
		## Handle Zero-Config MSI Uninstallations
		If ($useDefaultMsi) {
			[hashtable]$ExecuteDefaultMSISplat =  @{ Action = 'Uninstall'; Path = $defaultMsiFile }; If ($defaultMstFile) { $ExecuteDefaultMSISplat.Add('Transform', $defaultMstFile) }
			Execute-MSI @ExecuteDefaultMSISplat
		}
		
		
		##*===============================================
		##* POST-UNINSTALLATION
		##*===============================================
		[string]$installPhase = 'Post-Uninstallation'
		
		## <Perform Post-Uninstallation tasks here>
		
		Deregister-Installation
	}
	
	##*===============================================
	##* END SCRIPT BODY
	##*===============================================
	
	## Call the Exit-Script function to perform final cleanup operations
	Exit-Script -ExitCode $mainExitCode
}
Catch {
	[int32]$mainExitCode = 60001
	[string]$mainErrorMessage = "$(Resolve-Error)"
	Write-Log -Message $mainErrorMessage -Severity 3 -Source $deployAppScriptFriendlyName
	Show-DialogBox -Text $mainErrorMessage -Icon 'Stop'
	Exit-Script -ExitCode $mainExitCode
}